import json
import requests

def fetch_access_token(client_id, client_secret):
    auth_url = "https://app.4dlink.com/access/token"
    headers = {
        'Content-Type': 'application/json'
    }
    payload = json.dumps({
        "grant_type": "client_credentials",
        "client_id": client_id,
        "client_secret": client_secret
    })
    response = requests.post(auth_url, 
                             headers=headers, 
                             data=payload)

    if response.status_code == 200:
        response_json = response.json()
        access_token = response_json["payload"]["access_token"]
        return access_token

def fetch_projects(access_token):
    project_url = "https://app.4dlink.com/api/Project"
    headers = {
        'Authorization': f'Bearer {access_token}',
    }
    response = requests.get(f"{project_url}?limit=1", headers=headers)

    if response.status_code == 200:
        total_record = response.json()['total']
        response = requests.get(f"{project_url}?limit={total_record}", headers=headers)
        projects = response.json()['payload']
        project_names = []
        for project in projects:
            project_name = project.get('name')
            project_id = project.get('id')
            project_names.append(f"{project_id}:{project_name}")

    return project_names

def fetch_group_dataset_by_project(access_token, project_id):
    project_url = f"https://app.4dlink.com/api/Project/{project_id}?groups=true"
    headers = {
        'Authorization': f'Bearer {access_token}',
    }
    layer_group_dataset_tree = {}
    response = requests.get(project_url, headers=headers)
    if response.status_code == 200 and response.headers.get('content-type') == 'application/json':
        layer_groups = response.json()['payload']['grouped']
        for layer_group in layer_groups:
            layer_group_name = layer_group["name"]
            layer_group_id = layer_group["id"]
            datasets = layer_group["transforms"]
            dataset_details = []
            for dataset in datasets:
                dataset_id = dataset["id"]
                dataset_name = dataset["name"]
                dataset_detail = f"{dataset_id}:{dataset_name}"
                dataset_details.append(dataset_detail)
            layer_group_dataset_tree[f"{layer_group_id}:{layer_group_name}"] = dataset_details
    return layer_group_dataset_tree

def fetch_dataset_file(access_token, dataset_id):
    download_dataset_url = f"https://app.4dlink.com/api/Dataset/{dataset_id}/data"
    headers = {
        'Authorization': f'Bearer {access_token}',
        # 'Content-Type': 'application/binary'
    }
    response = requests.get(download_dataset_url, headers=headers)
    print(response.text)
    return response.content

def fetch_folder(access_token, select_layer_group_id):
    layer_group_url = "https://app.4dlink.com/api/Folder"
    headers = {
        'Authorization': f'Bearer {access_token}',
    }
    response = requests.get(f"{layer_group_url}?limit=1", headers=headers)
    if response.status_code == 200:
        total_record = response.json()['total']
        response = requests.get(f"{layer_group_url}?limit={total_record}", headers=headers)
        folders = response.json()['payload']
        for folder in folders:
            if folder.get("forScene") and folder.get("forScene").get("type") == 'TransformGroup':
                if int(select_layer_group_id) == int(folder.get("forScene").get("id")):
                    folder_id = folder.get("id")
                    return folder_id
                    break

def create_dataset(access_token, file_name, select_folder_id):
    dataset_url = "https://app.4dlink.com/api/Dataset"
    headers = {
        'Authorization': f'Bearer {access_token}',
        'Content-Type': 'application/json'
    }
    message_body = json.dumps({
        "name": file_name,
        "folder": {"id": select_folder_id}
    })
    response = requests.post(dataset_url, data=message_body, headers=headers)
    if response.status_code == 200:
        response_content = response.json()
        dataset_id = response_content.get("payload").get("id")
        return dataset_id
    else:
        print(response.json())